@@ -17,6 +17,9 @@
 #include <KLocalizedString>
 #include <KStandardShortcut>
 
+#include "kglobalaccel_component_interface.h"
+#include "kglobalaccel_interface.h"
+
 #include <QAbstractListModel>
 #include <QAction>
 #include <QDBusMetaType>
@@ -199,8 +202,8 @@ uint GlobalShortcutsPortal::CreateSession(const QDBusObjectPath &handle,
     qCDebug(XdgDesktopPortalKdeGlobalShortcuts) << "    app_id: " << app_id;
     qCDebug(XdgDesktopPortalKdeGlobalShortcuts) << "    options: " << options;
 
-    auto session = qobject_cast<GlobalShortcutsSession *>(Session::createSession(this, Session::GlobalShortcuts, app_id, session_handle.path()));
-    if (!session) {
+    auto session = new GlobalShortcutsSession(this, app_id, session_handle.path());
+    if (!session->isValid()) {
         return PortalResponse::OtherError;
     }
 
@@ -345,4 +348,168 @@ void GlobalShortcutsPortal::ConfigureShortcuts(const QDBusObjectPath &session_ha
     job->start();
 }
 
+
+GlobalShortcutsSession::GlobalShortcutsSession(QObject *parent, const QString &appId, const QString &path)
+    : Session(parent, appId, path)
+    , m_token(path.mid(path.lastIndexOf(u'/') + 1))
+    , m_globalAccelInterface(
+          new KGlobalAccelInterface(QStringLiteral("org.kde.kglobalaccel"), QStringLiteral("/kglobalaccel"), QDBusConnection::sessionBus(), this))
+    , m_component(new KGlobalAccelComponentInterface(m_globalAccelInterface->service(),
+                                                     m_globalAccelInterface->getComponent(componentName()).value().path(),
+                                                     m_globalAccelInterface->connection(),
+                                                     this))
+{
+    qDBusRegisterMetaType<KGlobalShortcutInfo>();
+    qDBusRegisterMetaType<QList<KGlobalShortcutInfo>>();
+    qDBusRegisterMetaType<QKeySequence>();
+    qDBusRegisterMetaType<QList<QKeySequence>>();
+
+    connect(m_globalAccelInterface,
+            &KGlobalAccelInterface::yourShortcutsChanged,
+            this,
+            [this](const QStringList &actionId, const QList<QKeySequence> &newKeys) {
+                Q_UNUSED(newKeys);
+                if (actionId[KGlobalAccel::ComponentUnique] == componentName()) {
+                    if (auto it = m_shortcuts.find(actionId[KGlobalAccel::ActionUnique]); it != m_shortcuts.end()) {
+                        it->second->setShortcuts(newKeys);
+                        Q_EMIT shortcutsChanged();
+                    }
+                }
+            });
+    connect(m_component,
+            &KGlobalAccelComponentInterface::globalShortcutPressed,
+            this,
+            [this](const QString &componentUnique, const QString &actionUnique, qlonglong timestamp) {
+                if (componentUnique != componentName()) {
+                    return;
+                }
+
+                Q_EMIT shortcutActivated(actionUnique, timestamp);
+            });
+    connect(m_component,
+            &KGlobalAccelComponentInterface::globalShortcutReleased,
+            this,
+            [this](const QString &componentUnique, const QString &actionUnique, qlonglong timestamp) {
+                if (componentUnique != componentName()) {
+                    return;
+                }
+
+                Q_EMIT shortcutDeactivated(actionUnique, timestamp);
+            });
+}
+
+GlobalShortcutsSession::~GlobalShortcutsSession() = default;
+
+void GlobalShortcutsSession::setActions(const QList<ShortcutInfo> &shortcuts)
+{
+    const QList<KGlobalShortcutInfo> shortcutInfos = m_component->allShortcutInfos();
+    QHash<QString, KGlobalShortcutInfo> shortcutInfosByName;
+    shortcutInfosByName.reserve(shortcutInfos.size());
+    for (const auto &shortcutInfo : shortcutInfos) {
+        shortcutInfosByName[shortcutInfo.uniqueName()] = shortcutInfo;
+    }
+
+    for (const auto &shortcut : shortcuts) {
+        if (shortcut.id.isEmpty()) {
+            qCWarning(XdgDesktopPortalKdeGlobalShortcuts) << "Shortcut without id" << "for" << componentName();
+            continue;
+        }
+        if (shortcut.description.isEmpty()) {
+            qCWarning(XdgDesktopPortalKdeGlobalShortcuts) << "Shortcut without name or description" << shortcut.id << "for" << componentName();
+            continue;
+        }
+
+        std::unique_ptr<QAction> &action = m_shortcuts[shortcut.id];
+        if (!action) {
+            action = std::make_unique<QAction>();
+        }
+        const QString app = Utils::applicationName(m_appId);
+        action->setProperty("componentName", componentName());
+        action->setProperty("componentDisplayName", app.isEmpty() ? componentName() : app);
+        action->setObjectName(shortcut.id);
+        action->setText(shortcut.description);
+        const auto itShortcut = shortcutInfosByName.constFind(shortcut.id);
+        if (itShortcut != shortcutInfosByName.constEnd()) {
+            action->setShortcuts(itShortcut->keys());
+        } else {
+            action->setShortcut(shortcut.keySequence);
+            if (!shortcut.preferredKeySequence.isEmpty()) {
+                KGlobalAccel::self()->setDefaultShortcut(action.get(), {shortcut.preferredKeySequence});
+            }
+        }
+        // If it's a new shortcut assign keySequence otherwise the old ones will be loaded.
+        KGlobalAccel::self()->setShortcut(action.get(), {shortcut.keySequence}, KGlobalAccel::Autoloading);
+
+        shortcutInfosByName.remove(shortcut.id);
+    }
+
+           // We can forget the shortcuts that aren't around anymore
+    while (!shortcutInfosByName.isEmpty()) {
+        const QString shortcutName = shortcutInfosByName.begin().key();
+        auto it = m_shortcuts.find(shortcutName);
+        if (it != m_shortcuts.end()) {
+            KGlobalAccel::self()->removeAllShortcuts(it->second.get());
+            m_shortcuts.erase(it);
+        }
+        shortcutInfosByName.erase(shortcutInfosByName.begin());
+    }
+
+    Q_ASSERT(static_cast<qsizetype>(m_shortcuts.size()) == shortcuts.size());
+}
+
+void GlobalShortcutsSession::loadActions()
+{
+    m_shortcuts.clear();
+
+    const QList<KGlobalShortcutInfo> shortcutInfos = m_component->allShortcutInfos();
+    QHash<QString, KGlobalShortcutInfo> shortcutInfosByName;
+    shortcutInfosByName.reserve(shortcutInfos.size());
+    for (const auto &shortcutInfo : shortcutInfos) {
+        shortcutInfosByName[shortcutInfo.uniqueName()] = shortcutInfo;
+    }
+
+    for (const auto &[name, info] : shortcutInfosByName.asKeyValueRange()) {
+        std::unique_ptr<QAction> &action = m_shortcuts[name];
+        if (!action) {
+            action = std::make_unique<QAction>();
+        }
+        action->setProperty("componentName", componentName());
+        action->setProperty("componentDisplayName", Utils::applicationName(m_appId));
+        action->setObjectName(name);
+        action->setText(info.friendlyName());
+        action->setShortcuts(info.keys());
+        // Explicitly load existing global shortcut setting
+        KGlobalAccel::self()->setShortcut(action.get(), action->shortcuts(), KGlobalAccel::Autoloading);
+    }
+}
+
+QVariant GlobalShortcutsSession::shortcutDescriptionsVariant() const
+{
+    QDBusArgument retVar;
+    retVar << shortcutDescriptions();
+    return QVariant::fromValue(retVar);
+}
+
+Shortcuts GlobalShortcutsSession::shortcutDescriptions() const
+{
+    Shortcuts ret;
+    for (const auto &[key, action] : m_shortcuts) {
+        QStringList triggers;
+        triggers.reserve(action->shortcuts().size());
+        const auto shortcuts = action->shortcuts();
+        for (const auto &shortcut : shortcuts) {
+            triggers += shortcut.toString(QKeySequence::NativeText);
+        }
+
+        ret.append({key,
+                    QVariantMap{
+                                {QStringLiteral("description"), action->text()},
+                                {QStringLiteral("trigger_description"), triggers.join(i18n(", "))},
+                                }});
+    }
+    Q_ASSERT(ret.size() == static_cast<qsizetype>(m_shortcuts.size()));
+    return ret;
+}
+
 #include "globalshortcuts.moc"
+#include "moc_globalshortcuts.cpp"
