@@ -26,41 +26,52 @@
 
 using namespace Qt::StringLiterals;
 
+struct UserDetails {
+private:
+    // This must never be public, all properties should be cached!
+    OrgFreedesktopAccountsUserInterface m_userInterface{QStringLiteral("org.freedesktop.Accounts"),
+                                                        QStringLiteral("/org/freedesktop/Accounts/User%1").arg(getuid()),
+                                                        QDBusConnection::systemBus()};
+
+public:
+    // Cache properties so we don't need to look them up multiple times for the same dialog
+    QString m_iconFile = m_userInterface.iconFile();
+    bool m_iconFileExists = QFileInfo::exists(m_iconFile);
+    QString m_realName = m_userInterface.realName();
+    QString m_userName = m_userInterface.userName();
+};
+
 UserInfoDialog::UserInfoDialog(const QString &reason, const QString &app_id, QObject *parent)
     : QuickDialog(parent)
+    , m_userDetails(std::make_unique<UserDetails>())
 {
-    QString ifacePath = QStringLiteral("/org/freedesktop/Accounts/User%1").arg(getuid());
-    m_userInterface = new OrgFreedesktopAccountsUserInterface(QStringLiteral("org.freedesktop.Accounts"), ifacePath, QDBusConnection::systemBus(), this);
-
     const KService::Ptr app = KService::serviceByDesktopName(app_id);
     const QString appName = app ? app->name() : app_id;
 
     QVariantMap props = {
-        {u"title"_s, i18n("Share User Information with %1", appName)},
-        {u"subtitle"_s, reason.isEmpty() ? i18n("%1 wants access to your user info.", appName) : reason},
-        {u"iconName"_s, QStringLiteral("user-identity")},
-        {u"details"_s, i18n("This app will be given access to your avatar and name.")},
-        {u"name"_s, m_userInterface->realName().isEmpty() ? m_userInterface->userName() : m_userInterface->realName()},
+        {u"mainText"_s, i18nc("@title", "Share user info with %1", appName)},
+        {u"subtitle"_s,
+         reason.isEmpty() ? i18nc("@info:usagetip", "Allows your username, full name, and profile picture to be used by the application.", appName) : reason},
+        {u"username"_s, id()},
+        {u"realname"_s, name()},
         {u"avatar"_s, image()},
     };
 
     create(u"UserInfoDialog"_s, props);
 }
 
-UserInfoDialog::~UserInfoDialog()
-{
-}
+UserInfoDialog::~UserInfoDialog() = default;
 
 QString UserInfoDialog::id() const
 {
-    return m_userInterface->userName();
+    return m_userDetails->m_userName;
 }
 
 QString UserInfoDialog::image() const
 {
     // TODO: Ideally plasma-welcome and kcm_users should always set an avatar (if need be to a render of Kirigami.Avatar)
     // such that the user always has a valid avatar 99.9% of the time.
-    if (!QFileInfo::exists(m_userInterface->iconFile())) {
+    if (!m_userDetails->m_iconFileExists) {
         // Always provide a fallback. We **must** provide an avatar to XDP per its documentation and code.
         // Legacy users may not have one, systems without Accounts service may not have one, some software may clear it...
         static const auto iconURI = [] {
@@ -71,10 +82,12 @@ QString UserInfoDialog::image() const
         //   but forward the original (symlink) path
         return iconURI;
     }
-    return QUrl::fromLocalFile(m_userInterface->iconFile()).toString();
+    return QUrl::fromLocalFile(m_userDetails->m_iconFile).toString();
 }
 
 QString UserInfoDialog::name() const
 {
-    return m_userInterface->realName();
+    return m_userDetails->m_realName;
 }
+
+#include "moc_userinfodialog.cpp"
